# !pyinstaller -F labelme_to_coco.py -i labelme_to_coco.ico
# chenxinfeng
# ------使用方法------
# 直接拖动文件夹到EXE中
import labelme2coco
import os
import sys
# set directory that contains labelme annotations and image files

def converter(labelme_folder):
    parent_folder, cur_folder = os.path.split(os.path.abspath(labelme_folder))
    
    # set path for coco json to be saved
    save_json_path = os.path.join(parent_folder, cur_folder+"_trainval.json")

    # convert labelme annotations to coco
    labelme2coco.convert(labelme_folder, save_json_path)

if __name__ == '__main__':
    n = len(sys.argv)
    if n == 1:
        folder = input("Select a folder >> ")
        if folder == None:
            exit()
        else:
            sys.argv.append(folder)
            
    print(sys.argv[1:])
    
    labelme_folder = sys.argv[1]
    converter(labelme_folder)
    print("Succeed")



#!/usr/bin/python
# !pyinstaller -F coco_split.py -i coco_split.ico
# chenxinfeng
# ------使用方法------
# 直接拖动json到EXE中
#
#python coco_split.py data/鼠关键点/trainval.json
import json
import argparse
import funcy
from sklearn.model_selection import train_test_split

parser = argparse.ArgumentParser(description='Splits COCO annotations file into training and test sets.')
parser.add_argument('annotations', metavar='coco_annotations', type=str,
                    help='Path to COCO annotations file.')
parser.add_argument('-s', dest='split', type=float, nargs='?', const=0.9,
                    help="A percentage of a split; a number in (0, 1)")
parser.add_argument('--having-annotations', dest='having_annotations', action='store_true',
                    help='Ignore all images without annotations. Keep only these with at least one annotation')

args = parser.parse_args()

def save_coco(file, info, licenses, images, annotations, categories):
    with open(file, 'wt', encoding='UTF-8') as coco:
        json.dump({ 'info': info, 'licenses': licenses, 'images': images, 
            'annotations': annotations, 'categories': categories}, coco, indent=2, sort_keys=True)

def filter_annotations(annotations, images):
    image_ids = funcy.lmap(lambda i: int(i['id']), images)
    return funcy.lfilter(lambda a: int(a['image_id']) in image_ids, annotations)

def main(args):
    if args.annotations.endswith('trainval.json'):
        args_train = args.annotations.replace('trainval.json', 'train.json')
        args_test  = args.annotations.replace('trainval.json', 'val.json')
    else:
        prefix =  os.path.splitext(args.annotations)[0] #without extension
        args_train = prefix+'_train.json'
        args_test = prefix+'_val.json'
        
    with open(args.annotations, 'rt', encoding='UTF-8') as annotations:
        coco = json.load(annotations)
        info = coco.get('info', '')
        licenses = ''
        images = coco['images']
        annotations = coco['annotations']
        categories = coco['categories']

        number_of_images = len(images)

        images_with_annotations = set(funcy.lmap(lambda a: int(a['image_id']), annotations))

        if args.having_annotations:
            images = funcy.lremove(lambda i: i['id'] not in images_with_annotations, images)

        x, y = train_test_split(images, train_size=args.split)

        save_coco(args_train, info, licenses, x, filter_annotations(annotations, x), categories)
        save_coco(args_test, info, licenses, y, filter_annotations(annotations, y), categories)

        print("Saved {} entries in {} and {} in {}".format(len(x), args_train, len(y), args_test))


if __name__ == "__main__":
    main(args)


#!/usr/bin/python
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import numpy as np
import argparse
import imageio
import time
import cv2
import os
import warnings
warnings.filterwarnings("ignore")
import json

from multiview_calib import utils 
from multiview_calib.bundle_adjustment_scipy import (build_input, bundle_adjustment, evaluate, 
                                                     visualisation, unpack_camera_params)
from multiview_calib.singleview_geometry import reprojection_error
from multiview_calib.extrinsics import verify_landmarks


def main(poses, landmarks, out, dump_images, setup, filenames): 
    poses = utils.json_read(poses)
    landmarks = utils.json_read(landmarks)
    views = list(landmarks.keys())
    intrinsics = {view:{'K':data['K'], 'dist':data['dist']} for view,data in poses.items()}
    extrinsics = {view:{'R':data['R'], 't':data['t']} for view,data in poses.items()}  
    camera_params, points_3d, points_2d,\
    camera_indices, point_indices, \
    n_cameras, n_points, ids, views_and_ids = build_input(views, intrinsics, extrinsics, 
                                                            landmarks, each=1, 
                                                            view_limit_triang=4)
    datadict = {'ids': np.array(ids).tolist(), 'points_3d': points_3d.tolist()}
    with open(out, 'w') as f:
        json.dump(datadict, f, indent=2)

    if dump_images:
        setup = utils.json_read(setup)
        filenames_images = utils.json_read(filenames)
        visualisation(setup, landmarks, filenames_images, 
                  camera_params, points_3d, 
                  points_2d, camera_indices, each=1, path='output/custom')  


if __name__ == "__main__":

    def str2bool(v):
        if v.lower() in ('yes', 'true', 't', 'y', '1'):
            return True
        elif v.lower() in ('no', 'false', 'f', 'n', '0'):
            return False
        else:
            raise argparse.ArgumentTypeError('Boolean value expected.')    

    parser = argparse.ArgumentParser()  
    parser.add_argument("--landmarks", "-l", type=str, required=True, default="landmarks.json",
                        help='JSON file containing the landmarks for each view')
    parser.add_argument("--poses", "-p", type=str, required=True, default="ba_poses.json",
                        help='JSON file containing the optimized poses')
    parser.add_argument("--out", "-o", type=str, required=True, default="out_points3d.json",
                        help='JSON file output for point3d')
    parser.add_argument("--filenames", "-f", type=str, required=False, default="filenames.json",
                        help='JSON file containing one filename of an image for each view. Used onyl if --dump_images is on')
    parser.add_argument("--dump_images", "-d", default=False, const=True, action='store_const',
                        help='Saves images for visualisation') 
    parser.add_argument("--setup", "-s", type=str, default=None,
                        help='setup.json') 
    args = parser.parse_args()

    main(**vars(args))



#!/usr/bin/python
'''
Author: your name
Date: 2021-09-28 14:14:27
LastEditTime: 2021-10-13 19:04:19
LastEditors: Please set LastEditors
Description: In User Settings Edit
FilePath: \BCNete:\cxf\crop_video.py
'''
# !pyinstaller -F crop_video.py -i mypython.ico
# chenxinfeng
# ------使用方法------
# 直接拖动文件夹到EXE中

'''
w, h = 800, 600
crop_xywh = [[w*0,h*0,w,h],
             [w*1,h*0,w,h],
             [w*2,h*0,w,h],
             [w*0,h*1,w,h],
             [w*1,h*1,w,h],
             [w*2,h*1,w,h]]
crop_tbg = ''
crop_tdur = '00:02:00'
'''
import sys
import os
from glob import glob 
import cv2
import platform

#xywh = [50,0,600,585]
#folder = r"E:\cxf\Videos\20210601 LS RAT"
crop_tbg = None
crop_tdur = None
def xywh2whxy(xywh, keepXeqY=True):
    if keepXeqY:
        maxXY = max(xywh[2:])
        xywh[2] = xywh[3] = maxXY
    whxy = (xywh[2], xywh[3], xywh[0], xywh[1])
    return whxy

def convert_folder_to_mp4(folder, whxy, postfix=None):
    os.chdir(folder)
    filenamesList = glob(r'*.avi') + glob(r'*.mp4') + glob(r'*.mkv')
    filenamesList = [f for f in filenamesList if 'output' not in f]
    
    # check filenameList is sorted. If not, rise an error!
    if len(filenamesList) == 0:
        print("Folder contain no AVI/MP4/MKV videos!")
        return
        
    # check FFMPEG exist
    ffmpeg_path = 'ffmpeg'

    ffmpeg_args = ''
    if crop_tbg:
        ffmpeg_args = ffmpeg_args + f' -ss {crop_tbg}'
    if crop_tdur:
        ffmpeg_args = ffmpeg_args + f' -t {crop_tdur}'
        
    # convert videos
    for filename in filenamesList:
        # check & downsample frame rate <=10fps
        cap= cv2.VideoCapture(filename)
        framespersecond= cap.get(cv2.CAP_PROP_FPS)
        cap.release()
        fps10 = min([10, framespersecond])
    
        # convert
        if platform.system()== 'Linux':
            codec = 'h264'
        else:
            codec = 'h264_nvenc'
        outfileformat = "{}_output_"+str(postfix)+".mp4" if postfix else "{}_output.mp4"
        outfilename = outfileformat.format(filename[:-4])
        mystr = '{} -i "{}" -vf crop={}:{}:{}:{} -r {} {} -c:v {} "{}" -y'.format(
            ffmpeg_path, filename, whxy[0], whxy[1], whxy[2], whxy[3], fps10, ffmpeg_args, codec, outfilename)
        print(mystr)
        out = os.system(mystr)

if __name__ == '__main__':
    n = len(sys.argv)
    if n == 1:
        folder = cg.uigetfolder()
        if folder == None:
            exit()
        else:
            sys.argv.append(folder)
            
    print(sys.argv[1:])

    video_foldname = os.path.abspath(sys.argv[1])
    config = cg.getfoldconfigpy(video_foldname)
    assert hasattr(config, 'crop_xywh'), 'Need [crop_xywh] in [config_video.py]'
    keepXeqY = getattr(config, 'keepXeqY', True)
    crop_tbg = getattr(config, 'crop_tbg', None)
    crop_tdur = getattr(config, 'crop_tdur', None)
    if type(config.crop_xywh[0]) == list:
        for i, crop_xywh in enumerate(config.crop_xywh):
            whxy = xywh2whxy(crop_xywh, keepXeqY)
            convert_folder_to_mp4(video_foldname, whxy, i+1)
    else:
        whxy = xywh2whxy(config.crop_xywh, keepXeqY)
        convert_folder_to_mp4(video_foldname, whxy)
    print("Succeed")
    



#!/usr/bin/python
'''
Author: your name
Date: 2021-11-13 19:17:10
LastEditTime: 2021-11-14 23:55:58
LastEditors: Please set LastEditors
Description: 打开koroFileHeader查看配置 进行设置: https://github.com/OBKoro1/koro1FileHeader/wiki/%E9%85%8D%E7%BD%AE
FilePath: \multiview_calib\examples\cxf_ball_rat\dlcBall_2_landmarks_ballglobal.py
'''
#python dlcBall_2_landmarks_ballglobal.py E:/cxf/dlc_ball_global

import json
import sys
import os
import glob
import pandas as pd
import numpy as np
views = ['c-1', 'c-2', 'c-3', 'c-4', 'c-5', 'c-6']

def view_link_csv(folder_csv):
    files_csv = glob.glob(os.path.join(folder_csv, '*.csv'))
    files_csv.sort()
    assert len(views)==len(files_csv)
    return dict(zip(views, files_csv))

def a1_load_dlc_csv(file):
    with open(file, 'r') as f:
        for iline, line in enumerate(f):
            if line.startswith('coords'):
                break
        else:
            raise 'Not Valid deeplabcut csv'
    df=pd.read_csv(file, header=list(range(iline)), skiprows=1)
    return df
    
def a2_flatten_points(df):
    arr = df
    for i in range(4):
        if 'x' in arr and 'y' in arr:
            break
        else:
            arr = arr.droplevel(level=0, axis=1)
    else:
        raise 'Not Valid deeplabcut csv'
    x = arr['x'].to_numpy()
    y = arr['y'].to_numpy()
    p = arr['likelihood'].to_numpy() if 'likelihood' in arr else (x*0+1.0)
    x_1v = x.flatten()
    y_1v = y.flatten()
    p_1v = p.flatten()
    return x_1v, y_1v, p_1v
    

def a3_thr_points(x_1v, y_1v, p_1v, thr=0.9):
    ids = np.arange(len(x_1v))
    idx_good = np.logical_not(np.isnan(p_1v))  & (p_1v > thr)
    px  = x_1v[idx_good]
    py  = y_1v[idx_good]
    ids = ids[idx_good]
    return px, py, ids
    
def a4_points_to_landmarksdict(px, py, ids):
    ids = ids.tolist()
    landmarks = np.array([px, py]).T.tolist()
    landmarksdict = {'ids':ids, 'landmarks':landmarks}
    return landmarksdict
    
def read_csv(file):
    df = a1_load_dlc_csv(file)
    x_1v, y_1v, p_1v = a2_flatten_points(df)
    px, py, ids = a3_thr_points(x_1v, y_1v, p_1v)
    landmarksdict = a4_points_to_landmarksdict(px, py, ids)
    return landmarksdict

def main(folder_csv, outjson):
    maps = view_link_csv(folder_csv)
    outdict = dict()
    for name, file in maps.items():
        print(name, os.path.split(file)[-1])
        outdict[name] = read_csv(file)

    with open(outjson, 'w') as f:
        json.dump(outdict, f, indent=4)
    
if __name__ == '__main__':
    n = len(sys.argv)
    if n == 1:
        folder = input("Choose the folder: >> ")
        if folder == None:
            exit()
        else:
            sys.argv.append(folder)
            
    print(sys.argv[1:])
    folder = sys.argv[1]
    assert sys.argv[2] in ['-o', '--out']
    outjson = sys.argv[3]
    main(folder, outjson)
    



##load all sub-modules
# dirname = os.path.dirname(__file__)
# for module in os.listdir(dirname):
	# if os.path.isfile(dirname+'/'+module):
		# if module == '__init__.py' or module[-3:] != '.py':
			# continue
		# __import__(module[:-3], fromlist=[dirname])
	# elif os.path.isfile(dirname+'/'+module + '/.__init__.py'):
		# __import__(module, locals(), globals())
	# else:
		# pass
# del module
#__import__('lilab.cvutils')
#__all__ = ['cvutils']

from .cvutils import *
from . import cvutils



#!/usr/bin/python
import os, glob, sys
import os.path
import importlib.util

#http://timgolden.me.uk/python/win32_how_do_i/browse-for-a-folder.html
def uigetfolder():
    path = input("select a folder >> ")
    return path
        

def ui_assert(eqtion, msg):
    if not eqtion:
        input("Error错误 ")
        sys.exit(1)

def getffmpeg():
    ffmpeg_path_d = r"D:\L_ffmpeg\ffmpeg\bin\ffmpeg.exe"
    ffmpeg_path_linux = '/usr/bin/ffmpeg'
    ffmpeg_path_loc = os.path.join(os.path.split(sys.argv[0])[0], "ffmpeg.exe")
    if os.path.isfile(ffmpeg_path_loc):
        ffmpeg_path = ffmpeg_path_loc
    elif os.path.isfile(ffmpeg_path_linux):
        ffmpeg_path = ffmpeg_path_linux
    elif os.path.isfile(ffmpeg_path_d):
        ffmpeg_path = ffmpeg_path_d
    else:
        ui_assert(False, "找不到FFMPEG.EXE, 请拷贝到程序文件夹!")
    return ffmpeg_path
        
def getfoldconfigpy(folder):
    configfile = os.path.join(folder, "config_video.py")
    assert os.path.isfile(configfile), 'No [{}] such config file'.format(configfile)
    spec=importlib.util.spec_from_file_location("config_video",configfile)
    foo = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(foo)
    return foo
    


#!/usr/bin/python
# !pyinstaller -F extract_frames.py -i mypython.ico
# chenxinfeng
# ------使用方法------
# 直接拖动文件夹/视频到EXE中

import os
import cv2
import numpy as np
import tqdm
import sys
import cxfguilib as cg
from glob import glob

numframe_to_extract = 200
maxlength = 10000
frame_dir = "outframes"

def extract(video_input, numframe_to_extract, maxlength):
    dirname,filename=os.path.split(video_input)
    nakefilename = os.path.splitext(filename)[0]
    cap = cv2.VideoCapture(video_input)
    os.makedirs(frame_dir, exist_ok = True)
    length = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
    length = min([maxlength, length-1])
    idxframe_to_extract = set(np.random.permutation(length)[:numframe_to_extract])
    idxframe_max = max(idxframe_to_extract)

    for iframe in tqdm.tqdm(range(length)):
        ret, frame = cap.read()
        if not ret: break
        if iframe>idxframe_max: break
        if iframe not in idxframe_to_extract: continue
        filename = os.path.join(dirname, frame_dir, nakefilename + '_{0:06}.jpg'.format(iframe))
        cv2.imwrite(filename, frame)
        
    cap.release()
    
if __name__ == '__main__':
    n = len(sys.argv)
    if n == 1:
        folder = cg.uigetfolder()
        if folder == None:
            exit()
        else:
            sys.argv.append(folder)
            
    print(sys.argv[1:])
    
    # check input is file or folder
    file_or_folderpath = sys.argv[1]
    if os.path.isfile(file_or_folderpath):
        video_foldname, f =os.path.split(file_or_folderpath)
        os.chdir(video_foldname)
        filenamesList = [f]
    elif os.path.isdir(file_or_folderpath):
        video_foldname = file_or_folderpath
        os.chdir(video_foldname)
        filenamesList = glob(r'*.avi') + glob(r'*.mp4') + glob(r'*.mkv')
        assert len(filenamesList), "Folder contain no AVI/MP4/MKV videos!"
    else:
        assert False, 'Input should be FILE or FOLDER'
        
    # read config_video.py
    try:
        config = cg.getfoldconfigpy('.')
        numframe_to_extract = getattr(config, 'numframe_to_extract', 200)
        maxlength = getattr(config, 'maxlength', 10000)
    except:
        pass
    
    for video_input in filenamesList:
        extract(video_input, numframe_to_extract, maxlength)
    
    print("Succeed")



import sys

import cv2  # Opencv ver 3.1.0 used
import numpy as np

# Set recursion limit
sys.setrecursionlimit(10 ** 5)

from opencvdragrect import selectinwindow

# paras
videofile = r"E:\cxf\2021-06-03 15-14-45 sub1p3.mp4"
MAXWIDTH, MAXHEIGHT = 1024, 800

cap = cv2.VideoCapture(videofile)
ret, frame = cap.read()
assert ret
cap.release()
frameHeight, frameWidth, _ = frame.shape
scale = max([frameWidth/MAXWIDTH, frameHeight/MAXHEIGHT, 1])



# Initialize the  drag object
wName = "select region"
imageWidth = int(frameWidth / scale)
imageHeight = int(frameHeight / scale)
image = frame

# Define the drag object
rectI = selectinwindow.DragRectangle(image, wName, frameWidth, frameHeight)

cv2.namedWindow(rectI.wname)
cv2.resizeWindow(rectI.wname, imageWidth, imageHeight);
cv2.setMouseCallback(rectI.wname, selectinwindow.dragrect, rectI)

# keep looping until rectangle finalized
while True:
    # display the image
    cv2.imshow(rectI.wname, rectI.image)
    if cv2.waitKey(1) in (ord('q'), 27): break
    # if returnflag is True, break from the loop
    if rectI.returnflag: break

print("Dragged rectangle coordinates")
print(str(rectI.outRect.x) + ',' + str(rectI.outRect.y) + ',' + \
      str(rectI.outRect.w) + ',' + str(rectI.outRect.h))

# close all open windows
cv2.destroyAllWindows()




#!/usr/bin/python
# !pyinstaller -F extract_frames.py -i mypython.ico
# chenxinfeng
# ------使用方法------
# 直接拖动文件夹/视频到EXE中

import os
import cv2
import numpy as np
import tqdm
import sys
from glob import glob

frame_dir = "outframes"

def timestr_to_seconds(frames_time):
    frames_seconds = []
    for frame_time in frames_time:
        if type(frame_time)==str:
            frame_seconds = sum(x * int(t) for x, t in zip([3600, 60, 1], frame_time.split(":")))
        else:
            frame_seconds = frame_time
        frames_seconds.append(frame_seconds)
    return frames_seconds    

def extract(video_input, frame_seconds):
    dirname,filename=os.path.split(video_input)
    nakefilename = os.path.splitext(filename)[0]
    cap = cv2.VideoCapture(video_input)
    fps = cap.get(cv2.CAP_PROP_FPS)
    os.makedirs(os.path.join(dirname, frame_dir),exist_ok=True)

    for iframe, frame_seconds in enumerate(tqdm.tqdm(frame_seconds)):
        cap.set(cv2.CAP_PROP_POS_FRAMES, int(frame_seconds * fps))
        ret, frame = cap.read()
        if not ret: continue
        filename = os.path.join(dirname, frame_dir, 'frame' + '_{0:02}.jpg'.format(iframe))
        cv2.imwrite(filename, frame)
        
    cap.release()
    
if __name__ == '__main__':
    n = len(sys.argv)
    if n == 1:
        folder = input("Choose video folder: >> ")
        if folder == None:
            exit()
        else:
            sys.argv.append(folder)
            
    print(sys.argv[1:])
    
    # check input is file or folder
    file_or_folderpath = sys.argv[1]
    if os.path.isfile(file_or_folderpath):
        video_foldname, f =os.path.split(file_or_folderpath)
        os.chdir(video_foldname)
        filenamesList = [f]
    elif os.path.isdir(file_or_folderpath):
        video_foldname = file_or_folderpath
        os.chdir(video_foldname)
        filenamesList = glob(r'*.avi') + glob(r'*.mp4') + glob(r'*.mkv')
        assert len(filenamesList), "Folder contain no AVI/MP4/MKV videos!"
    else:
        assert False, 'Input should be FILE or FOLDER'
        
    # read config_video.py
    import sys; sys.path.append('.')
    import config_video as config
    frames_time = getattr(config, 'frames_time', None)
    frames_seconds = timestr_to_seconds(frames_time)
    
    for video_input in filenamesList:
        extract(video_input, frames_seconds)
    
    print("Succeed")



import glob
import os
from PIL import Image

def concat(folder_1, folder_2):
    folder_out = folder_1 + '_x_' + os.path.split(folder_2)[-1]
    os.makedirs(folder_out, exist_ok=True)

    def globimages(folder):
        ret =   glob.glob(os.path.join(folder, '*.jpg')) + \
                glob.glob(os.path.join(folder, '*.jpeg')) + \
                glob.glob(os.path.join(folder, '*.jpeg'))
        return ret

    def commonimages(imgs1, imgs2):
        imgsNake1 = [os.path.split(img)[-1] for img in imgs1]
        imgsNake2 = [os.path.split(img)[-1] for img in imgs2]
        imgsNakeCommon = set(imgsNake1) & set(imgsNake2)
        imgsCommon1 = [img for img, imgNake in zip(imgs1, imgsNake1)
                             if imgNake in imgsNakeCommon]
        imgsCommon2 = [img for img, imgNake in zip(imgs2, imgsNake2)
                             if imgNake in imgsNakeCommon]
        return imgsCommon1, imgsCommon2

    imgs1 = globimages(folder_1)
    imgs2 = globimages(folder_2)
    imgs1, imgs2 = commonimages(imgs1, imgs2)

    for img1, img2 in zip(imgs1, imgs2):
        images = [Image.open(x) for x in [img1, img2]]
        widths, heights = zip(*(i.size for i in images))

        total_width = sum(widths)
        max_height = max(heights)

        new_im = Image.new('RGB', (total_width, max_height))

        x_offset = 0
        for im in images:
            new_im.paste(im, (x_offset,0))
            x_offset += im.size[0]

        new_im.save(os.path.join(folder_out, os.path.split(img1)[-1]))
    
if __name__ == '__main__':
    folder_1 = input("(1/2)The left  folder: >>")
    folder_2 = input("(2/2)The right folder: >>")
    concat(folder_1, folder_2)


# import os
# import glob

##load all sub-modules
# dirname = os.path.dirname(__file__)
# for module in os.lisdir(dirname):
	# if os.path.isfile(module):
		# if module == '__init__.py' or module[-3:] != '.py':
			# continue
		# __import__(module[:-3], locals(), globals())
	# elif os.path.isdir(module) and os.path.isfile(module+'/.__init__.py'):
		# __import__(module, locals(), globals())
	# else:
		# pass
# del module
from . import cxfguilib, crop_video, dlcBall_2_landmarks
    
#,
               # crop_video, cxfguilib, dlcBall_2_landmarks, draw_video_rect,
               # extract_firstframe_toviews, extract_frames, extract_frames_by_time,
               # extract_JPG_from_dataset, labelme_to_coco, landmarks2point3d,
               # merge_multipart_coco, , coco_split, coco_viewer, concat_image, crop_image



# !pyinstaller -F coco_viewer.py -i coco_viewer.ico
# chenxinfeng
# ------使用方法------
# 直接拖动文件夹到EXE中
"""COCO Dataset Viewer.

View images with bboxes from the COCO dataset.
"""
import argparse
import os
import random
import colorsys
import json
import logging
import tkinter as tk
import tkinter.ttk as ttk
from tkinter import filedialog, messagebox
from turtle import __forwardmethods

from PIL import Image, ImageDraw, ImageTk, ImageFont

logging.basicConfig(level=logging.INFO, format='%(levelname)s: %(message)s')

parser = argparse.ArgumentParser(description="View images with bboxes from the COCO dataset")
parser.add_argument("annotations", type=str, help="path to annotations json file")
parser.add_argument("--images", default='', type=str, help="path to images folder")


class Data:
    """Handles data related stuff.
    """
    def __init__(self, image_dir, annotations_file):
        self.image_dir = image_dir
        instances, images, categories = parse_coco(annotations_file)
        self.instances = instances
        self.images = ImageList(images)  # NOTE: image list is based on annotations file
        self.categories = categories  # Dataset categories

        # Prepare the very first image
        self.current_image = self.images.next()  # Set the first image as current

    def prepare_image(self):
        """Prepares image path, objects, colors.
        """
        # TODO: predicted bboxes drawing (from models)
        img_id, img_name = self.current_image
        full_path = os.path.join(self.image_dir, img_name)

        # Get objects and category ids
        objects = [obj for obj in self.instances["annotations"] if obj["image_id"] == img_id]
        obj_categories_ids = [obj["category_id"] for obj in objects]

        # List of category ids of all objects
        img_obj_categories = [obj["category_id"] for obj in objects]
        # Current image categories (unique sorted category ids)
        img_categories = sorted(list(set(img_obj_categories)))

        # Get category name-color pairs for the objects
        names_colors = [self.categories[i] for i in obj_categories_ids]

        return full_path, objects, names_colors, img_obj_categories, img_categories

    def next_image(self):
        """Loads the next image in a list.
        """
        self.current_image = self.images.next()

    def previous_image(self):
        """Loads the previous image in a list.
        """
        self.current_image = self.images.prev()


def parse_coco(annotations_file: str) -> tuple:
    """Parses COCO json annotation file.
    """
    instances = load_annotations(annotations_file)
    images = get_images(instances)
    categories = get_categories(instances)
    return instances, images, categories


def load_annotations(fname: str) -> dict:
    """Loads annotations file.
    """
    logging.info(f"Parsing {fname}...")

    with open(fname) as f:
        instances = json.load(f)
    return instances


def get_images(instances: dict) -> list:
    """Extracts all image ids and file names from annotations file.
    """
    return [(image["id"], image["file_name"]) for image in instances["images"]]


def open_image(full_img_path: str):
    """Opens image, creates draw context.
    """
    # Open image
    img_open = Image.open(full_img_path).convert("RGBA")
    # Create layer for bboxes and masks
    draw_layer = Image.new("RGBA", img_open.size, (255, 255, 255, 0))
    draw = ImageDraw.Draw(draw_layer)
    return img_open, draw_layer, draw


def get_categories(instances: dict) -> dict:
    """Extracts categories from annotations file and prepares color for each one.
    """
    # Get some colors
    # hsv_tuples = [(x / 80, 1., 1.) for x in range(80)]
    # colors = list(map(lambda x: colorsys.hsv_to_rgb(*x), hsv_tuples))
    # colors = list(map(lambda x: (int(x[0] * 255), int(x[1] * 255), int(x[2] * 255)), colors))
    # random.seed(42)
    # random.shuffle(colors)
    # random.seed(None)
    colors = [(31, 120, 180), (51, 160, 44), (227, 26, 28), (255, 127, 0)]
    # Parse categories
    categories = list(zip([[category["id"], category["name"]] for category in instances["categories"]], colors))
    categories = dict([[cat[0][0], [cat[0][1], cat[1]]] for cat in categories])
    return categories


def draw_bboxes(draw, objects, labels, obj_categories, ignore, width, label_size):
    """Puts rectangles on the image.
    """
    # Extracting bbox coordinates
    bboxes = [[obj["bbox"][0],
               obj["bbox"][1],
               obj["bbox"][0] + obj["bbox"][2],
               obj["bbox"][1] + obj["bbox"][3]] for obj in objects]
    # Draw bboxes
    for i, (c, b) in enumerate(zip(obj_categories, bboxes)):
        if i not in ignore:
            draw.rectangle(b, outline=c[-1], width=width)

            if labels:
                text = c[0]
                font = ImageFont.truetype("arial.ttf", size=label_size)

                tw, th = draw.textsize(text, font)
                tx0 = b[0]
                ty0 = b[1] - th

                # TODO: Looks weird! We need image dims to make it right
                tx0 = max(b[0], max(b[0], tx0)) if tx0 < 0 else tx0
                ty0 = max(b[1], max(0, ty0)) if ty0 < 0 else ty0

                tx1 = tx0 + tw
                ty1 = ty0 + th

                # TODO: The same here
                if tx1 > b[2]:
                    tx0 = max(0, tx0 - (tx1 - b[2]))
                    tx1 = tw if tx0 == 0 else b[2]

                draw.rectangle((tx0, ty0, tx1, ty1), fill=c[-1])
                draw.text((tx0, ty0), text, (255, 255, 255), font=font)


def draw_masks(draw, objects, obj_categories, ignore, alpha):
    """Draws a masks over image.
    """
    masks = [obj["segmentation"] for obj in objects]
    # Draw masks
    for i, (c, m) in enumerate(zip(obj_categories, masks)):
        if i not in ignore:
            alpha = alpha
            fill = tuple(list(c[-1]) + [alpha])
            # Polygonal masks work fine
            if isinstance(m, list):
                for m_ in m:
                    if m_:
                        draw.polygon(m_, outline=fill, fill=fill)
            # TODO: Fix problem with RLE
            # elif isinstance(m, dict):
            #     draw.polygon(m['counts'][1:-2], outline=c[-1], fill=fill)
            else:
                continue


class ImageList:
    """Handles iterating through the images.
    """
    def __init__(self, images: list):
        self.image_list = images or []
        self.n = -1
        self.max = len(self.image_list)

    def next(self):
        """Sets the next image as current.
        """
        self.n += 1

        if self.n < self.max:
            current_image = self.image_list[self.n]
        else:
            self.n = 0
            current_image = self.image_list[self.n]
        return current_image

    def prev(self):
        """Sets the previous image as current.
        """
        if self.n == 0:
            self.n = self.max - 1
            current_image = self.image_list[self.n]
        else:
            self.n -= 1
            current_image = self.image_list[self.n]
        return current_image


class ImagePanel(ttk.Frame):
    """ttk port of original turtle.ScrolledCanvas code.
    """
    def __init__(self, parent, width=768, height=480, canvwidth=600, canvheight=500):
        super().__init__(parent, width=width, height=height)
        self._rootwindow = self.winfo_toplevel()
        self.width, self.height = width, height
        self.canvwidth, self.canvheight = canvwidth, canvheight
        self.bg = "gray15"
        self.pack(fill=tk.BOTH, expand=True)

        self._canvas = tk.Canvas(parent, width=width, height=height, bg=self.bg, relief="sunken", borderwidth=2)
        self.hscroll = ttk.Scrollbar(parent, command=self._canvas.xview, orient=tk.HORIZONTAL)
        self.vscroll = ttk.Scrollbar(parent, command=self._canvas.yview)
        self._canvas.configure(xscrollcommand=self.hscroll.set, yscrollcommand=self.vscroll.set)

        self.rowconfigure(0, weight=1, minsize=0)
        self.columnconfigure(0, weight=1, minsize=0)
        self._canvas.grid(padx=1, in_=self, pady=1, row=0, column=0, rowspan=1, columnspan=1, sticky=tk.NSEW)
        self.vscroll.grid(padx=1, in_=self, pady=1, row=0, column=1, rowspan=1, columnspan=1, sticky=tk.NSEW)
        self.hscroll.grid(padx=1, in_=self, pady=1, row=1, column=0, rowspan=1, columnspan=1, sticky=tk.NSEW)

        self.reset()
        self._rootwindow.bind("<Configure>", self.on_resize)

    def reset(self, canvwidth=None, canvheight=None, bg=None):
        """Adjusts canvas and scrollbars according to given canvas size.
        """
        if canvwidth:
            self.canvwidth = canvwidth
        if canvheight:
            self.canvheight = canvheight
        if bg:
            self.bg = bg
        self._canvas.config(
            bg=bg,
            scrollregion=(
                -self.canvwidth // 2,
                -self.canvheight // 2,
                self.canvwidth // 2,
                self.canvheight // 2,
            ),
        )
        self._canvas.xview_moveto(0.5 * (self.canvwidth - self.width + 30) / self.canvwidth)
        self._canvas.yview_moveto(0.5 * (self.canvheight - self.height + 30) / self.canvheight)
        self.adjust_scrolls()

    def adjust_scrolls(self):
        """Adjusts scrollbars according to window- and canvas-size.
        """
        cwidth = self._canvas.winfo_width()
        cheight = self._canvas.winfo_height()

        self._canvas.xview_moveto(0.5 * (self.canvwidth - cwidth) / self.canvwidth)
        self._canvas.yview_moveto(0.5 * (self.canvheight - cheight) / self.canvheight)

        if cwidth < self.canvwidth:
            self.hscroll.grid(padx=1, in_=self, pady=1, row=1, column=0, rowspan=1, columnspan=1, sticky=tk.NSEW)
        else:
            self.hscroll.grid_forget()
        if cheight < self.canvheight:
            self.vscroll.grid(padx=1, in_=self, pady=1, row=0, column=1, rowspan=1, columnspan=1, sticky=tk.NSEW)
        else:
            self.vscroll.grid_forget()

    def on_resize(self, event):
        self.adjust_scrolls()

    def bbox(self, *args):
        return self._canvas.bbox(*args)

    def cget(self, *args, **kwargs):
        return self._canvas.cget(*args, **kwargs)

    def config(self, *args, **kwargs):
        self._canvas.config(*args, **kwargs)

    def bind(self, *args, **kwargs):
        self._canvas.bind(*args, **kwargs)

    def unbind(self, *args, **kwargs):
        self._canvas.unbind(*args, **kwargs)

    def focus_force(self):
        self._canvas.focus_force()


__forwardmethods(ImagePanel, tk.Canvas, "_canvas")


class StatusBar(ttk.Frame):
    """Shows status line on the bottom.
    """
    def __init__(self, parent):
        super().__init__(parent)
        # self.configure(bd="gray75")
        self.pack(side=tk.BOTTOM, fill=tk.X)

        self.file_count = ttk.Label(self, borderwidth=5, background="gray75")
        self.file_count.pack(side=tk.RIGHT)
        self.description = ttk.Label(self, borderwidth=5, background="gray75")
        self.description.pack(side=tk.RIGHT)
        self.file_name = ttk.Label(self, borderwidth=5, background="gray75")
        self.file_name.pack(side=tk.LEFT)
        self.nobjects = ttk.Label(self, borderwidth=5, background="gray75")
        self.nobjects.pack(side=tk.LEFT)
        self.ncategories = ttk.Label(self, borderwidth=5, background="gray75")
        self.ncategories.pack(side=tk.LEFT)


class Menu(tk.Menu):
    def __init__(self, parent):
        super().__init__(parent)
        # Define menu structure
        self.file = self.file_menu()
        self.view = self.view_menu()

    def file_menu(self):
        """File Menu.
        """
        menu = tk.Menu(self, tearoff=False)
        menu.add_command(label="Save", accelerator="Ctrl+S")
        menu.add_separator()
        menu.add_command(label="Exit", accelerator="Ctrl+Q")
        self.add_cascade(label="File", menu=menu)
        return menu

    def view_menu(self):
        """View Menu.
        """
        menu = tk.Menu(self, tearoff=False)
        menu.add_checkbutton(label="BBoxes", onvalue=True, offvalue=False)
        menu.add_checkbutton(label="Labels", onvalue=True, offvalue=False)
        menu.add_checkbutton(label="Masks", onvalue=True, offvalue=False)
        self.add_cascade(label="View", menu=menu)
        return menu


class ObjectsPanel(ttk.PanedWindow):
    """Panels with listed objects and categories for the image.
    """
    def __init__(self, parent):
        super().__init__(parent)
        self.pack(side=tk.RIGHT, fill=tk.Y)

        # Categories subpanel
        self.category_subpanel = ttk.Frame()
        ttk.Label(self.category_subpanel, text="categories", borderwidth=2, background="gray50").pack(side=tk.TOP, fill=tk.X)
        self.category_box = tk.Listbox(self.category_subpanel, selectmode=tk.EXTENDED, exportselection=0)
        self.category_box.pack(side=tk.TOP, fill=tk.Y, expand=True)
        self.add(self.category_subpanel)

        # Objects subpanel
        self.object_subpanel = ttk.Frame()
        ttk.Label(self.object_subpanel, text="objects", borderwidth=2, background="gray50").pack(side=tk.TOP, fill=tk.X)
        self.object_box = tk.Listbox(self.object_subpanel, selectmode=tk.EXTENDED, exportselection=0)
        self.object_box.pack(side=tk.TOP, fill=tk.Y, expand=True)
        self.add(self.object_subpanel)


class SlidersBar(ttk.Frame):
    def __init__(self, parent):
        super().__init__(parent)
        self.pack(side=tk.BOTTOM, fill=tk.X)

        # Bbox thickness controller
        self.bbox_slider = tk.Scale(self, label="bbox", from_=0, to=25, tickinterval=5, orient=tk.HORIZONTAL)
        self.bbox_slider.pack(side=tk.LEFT, fill=tk.X, expand=True)

        # Label text size controller
        self.label_slider = tk.Scale(self, label="label", from_=10, to=100, tickinterval=25, orient=tk.HORIZONTAL)
        self.label_slider.pack(side=tk.LEFT, fill=tk.X, expand=True)

        # Mask transparency controller
        self.mask_slider = tk.Scale(self, label="mask", from_=0, to=255, tickinterval=50, orient=tk.HORIZONTAL)
        self.mask_slider.pack(side=tk.LEFT, fill=tk.X, expand=True)


class Controller:
    def __init__(self, data, root, image_panel, statusbar, menu, objects_panel, sliders):
        self.data = data  # data layer
        self.root = root  # root window
        self.image_panel = image_panel  # image panel
        self.statusbar = statusbar  # statusbar on the bottom
        self.menu = menu  # main menu on the top
        self.objects_panel = objects_panel
        self.sliders = sliders

        # StatusBar Vars
        self.file_count_status = tk.StringVar()
        self.file_name_status = tk.StringVar()
        self.description_status = tk.StringVar()
        self.nobjects_status = tk.StringVar()
        self.ncategories_status = tk.StringVar()
        self.statusbar.file_count.configure(textvariable=self.file_count_status)
        self.statusbar.file_name.configure(textvariable=self.file_name_status)
        self.statusbar.description.configure(textvariable=self.description_status)
        self.statusbar.nobjects.configure(textvariable=self.nobjects_status)
        self.statusbar.ncategories.configure(textvariable=self.ncategories_status)

        # Menu Vars
        self.bboxes_on_global = tk.BooleanVar()  # Toggles bboxes globally
        self.bboxes_on_global.set(True)
        self.labels_on_global = tk.BooleanVar()  # Toggles category labels
        self.labels_on_global.set(True)
        self.masks_on_global = tk.BooleanVar()  # Toggles masks globally
        self.masks_on_global.set(True)
        # Menu Configuration
        self.menu.file.entryconfigure("Save", command=self.save_image)
        self.menu.file.entryconfigure("Exit", command=self.exit)
        self.menu.view.entryconfigure("BBoxes", variable=self.bboxes_on_global, command=self.menu_view_bboxes)
        self.menu.view.entryconfigure("Labels", variable=self.labels_on_global, command=self.menu_view_labels)
        self.menu.view.entryconfigure("Masks", variable=self.masks_on_global, command=self.menu_view_masks)
        self.root.configure(menu=self.menu)

        # Init local setup (for the current (active) image)
        self.bboxes_on_local = self.bboxes_on_global.get()
        self.labels_on_local = self.labels_on_global.get()
        self.masks_on_local = self.masks_on_global.get()

        # Objects Panel stuff
        self.selected_cats = None
        self.selected_objs = None
        self.category_box_content = tk.StringVar()
        self.object_box_content = tk.StringVar()
        self.objects_panel.category_box.configure(listvariable=self.category_box_content)
        self.objects_panel.object_box.configure(listvariable=self.object_box_content)

        # Sliders Setup
        self.bbox_thickness = tk.IntVar()
        self.bbox_thickness.set(3)
        self.label_size = tk.IntVar()
        self.label_size.set(15)
        self.mask_alpha = tk.IntVar()
        self.mask_alpha.set(128)
        self.sliders.bbox_slider.configure(variable=self.bbox_thickness, command=lambda e: self.update_img())
        self.sliders.label_slider.configure(variable=self.label_size, command=lambda e: self.update_img())
        self.sliders.mask_slider.configure(variable=self.mask_alpha, command=lambda e: self.update_img())

        # Bind all events
        self.bind_events()

        # Compose the very first image
        self.current_composed_image = None
        self.current_img_obj_categories = None
        self.current_img_categories = None
        self.update_img()

    def set_locals(self):
        self.bboxes_on_local = self.bboxes_on_global.get()
        self.labels_on_local = self.labels_on_global.get()
        self.masks_on_local = self.masks_on_global.get()

        # Update sliders
        self.update_sliders_state()

    def compose_image(
            self,
            full_path,
            objects,
            names_colors,
            bboxes_on: bool = True,
            labels_on: bool = True,
            masks_on: bool = True,
            ignore: list = None,
            width: int = 1,
            alpha: int = 128,
            label_size: int = 15,
    ):
        ignore = ignore or []  # list of objects to ignore
        img_open, draw_layer, draw = open_image(full_path)
        # Draw masks
        if masks_on:
            draw_masks(draw, objects, names_colors, ignore, alpha)
        # Draw bounding boxes
        if bboxes_on:
            draw_bboxes(draw, objects, labels_on, names_colors, ignore, width, label_size)
        del draw
        # Resulting image
        self.current_composed_image = Image.alpha_composite(img_open, draw_layer)

    def update_img(self, local=True, width=None, alpha=None, label_size=None):
        """Triggers image composition and sets composed image as current.
        """
        bboxes_on = self.bboxes_on_local if local else self.bboxes_on_global.get()
        labels_on = self.labels_on_local if local else self.labels_on_global.get()
        masks_on = self.masks_on_local if local else self.masks_on_global.get()

        # Prepare image
        full_path, objects, names_colors, img_obj_categories, img_categories = self.data.prepare_image()
        self.current_img_obj_categories = img_obj_categories
        self.current_img_categories = img_categories

        if self.selected_objs is None:
            ignore = []
        else:
            ignore = [i for i in range(len(self.current_img_obj_categories)) if i not in self.selected_objs]

        width = self.bbox_thickness.get() if width is None else width
        alpha = self.mask_alpha.get() if alpha is None else alpha
        label_size = self.label_size.get() if label_size is None else label_size

        # Compose image
        self.compose_image(
            full_path=full_path,
            objects=objects,
            names_colors=names_colors,
            bboxes_on=bboxes_on,
            labels_on=labels_on,
            masks_on=masks_on,
            ignore=ignore,
            width=width,
            alpha=alpha,
            label_size=label_size,
        )

        # Prepare PIL image for Tkinter
        img = self.current_composed_image
        w, h = img.size
        img = ImageTk.PhotoImage(img)

        # Set image as current
        self.image_panel.create_image(0, 0, image=img)
        self.image_panel.image = img
        self.image_panel.reset(canvwidth=w, canvheight=h)

        # Update statusbar vars
        self.file_count_status.set(f"{str(self.data.images.n + 1)}/{self.data.images.max}")
        self.file_name_status.set(f"{self.data.current_image[-1]}")
        self.description_status.set(f"{self.data.instances.get('info', dict()).get('description', '')}")
        self.nobjects_status.set(f"objects: {len(self.current_img_obj_categories)}")
        self.ncategories_status.set(f"categories: {len(self.current_img_categories)}")

        # Update Objects panel
        self.update_category_box()
        self.update_object_box()

    def exit(self, event=None):
        print_info("Exiting...")
        self.root.quit()

    def next_img(self, event=None):
        self.data.next_image()
        self.set_locals()
        self.selected_cats = None
        self.selected_objs = None
        self.update_img(local=False)

    def prev_img(self, event=None):
        self.data.previous_image()
        self.set_locals()
        self.selected_cats = None
        self.selected_objs = None
        self.update_img(local=False)

    def save_image(self, event=None):
        """Saves composed image as png file.
        """
        # Initial (original) file name
        initialfile = self.data.current_image[-1].split(".")[0]
        # TODO: Add more formats, at least jpg (RGBA -> RGB)?
        filetypes = (("png files", "*.png"), ("all files", "*.*"))
        # By default save as png file
        defaultextension = ".png"
        file = filedialog.asksaveasfilename(
            initialfile=initialfile,
            filetypes=filetypes,
            defaultextension=defaultextension,
        )
        # If not canceled:
        if file:
            self.data.current_composed_image.save(file)

    def menu_view_bboxes(self):
        self.bboxes_on_local = self.bboxes_on_global.get()
        self.bbox_slider_status_update()
        self.update_img()

    def menu_view_labels(self):
        self.labels_on_local = self.labels_on_global.get()
        self.label_slider_status_update()
        self.update_img()

    def menu_view_masks(self):
        self.masks_on_local = self.masks_on_global.get()
        self.masks_slider_status_update()
        self.update_img()

    def toggle_bboxes(self, event=None):
        self.bboxes_on_local = not self.bboxes_on_local
        self.bbox_slider_status_update()
        self.update_img()

    def toggle_labels(self, event=None):
        self.labels_on_local = not self.labels_on_local
        self.label_slider_status_update()
        self.update_img()

    def toggle_masks(self, event=None):
        self.masks_on_local = not self.masks_on_local
        self.update_img()

    def toggle_all(self, event=None):
        # Toggle only when focused on image
        if event.widget.focus_get() is self.objects_panel.category_box:
            return
        if event.widget.focus_get() is self.objects_panel.object_box:
            return
        # What to toggle
        var_list = [self.bboxes_on_local, self.labels_on_local, self.masks_on_local]
        # if any is on, turn them off
        if True in set(var_list):
            self.bboxes_on_local = False
            self.labels_on_local = False
            self.masks_on_local = False
        # if all is off, turn them on
        else:
            self.bboxes_on_local = True
            self.labels_on_local = True
            self.masks_on_local = True

        # Update sliders
        self.update_sliders_state()
        # Update image with updated vars
        self.update_img()

    def update_category_box(self):
        ids = self.current_img_categories
        names = [self.data.categories[i][0] for i in ids]
        self.category_box_content.set([" ".join([str(i), str(n)]) for i, n in zip(ids, names)])
        self.objects_panel.category_box.selection_clear(0, tk.END)
        if self.selected_cats is not None:
            for i in self.selected_cats:
                self.objects_panel.category_box.select_set(i)
        else:
            self.objects_panel.category_box.select_set(0, tk.END)

    def select_category(self, event):
        # Get selection from user
        selected_ids = self.objects_panel.category_box.curselection()
        # Set selected_cats
        self.selected_cats = selected_ids
        # Set selected_objs
        selected_objs = []
        for ci in self.selected_cats:
            for i, o in enumerate(self.current_img_obj_categories):
                if self.current_img_categories[ci] == o:
                    selected_objs.append(i)
        self.selected_objs = selected_objs
        self.update_img()

    def update_object_box(self):
        ids = self.current_img_obj_categories
        names = [self.data.categories[i][0] for i in ids]
        self.object_box_content.set([" ".join([str(i), str(n)]) for i, n in enumerate(names)])
        self.objects_panel.object_box.selection_clear(0, tk.END)
        if self.selected_objs is not None:
            for i in self.selected_objs:
                self.objects_panel.object_box.select_set(i)
        else:
            self.objects_panel.object_box.select_set(0, tk.END)

    def select_object(self, event):
        # Get selection from user
        selected_ids = self.objects_panel.object_box.curselection()
        # Set selected_cats
        self.selected_objs = selected_ids
        # Set selected_objs
        selected_cats = []
        for oi in self.selected_objs:
            for i, c in enumerate(self.current_img_categories):
                if self.current_img_obj_categories[oi] == c:
                    selected_cats.append(i)
        self.selected_cats = selected_cats
        self.update_img()

    def update_sliders_state(self):
        self.bbox_slider_status_update()
        self.label_slider_status_update()
        self.masks_slider_status_update()

    def bbox_slider_status_update(self):
        self.sliders.bbox_slider.configure(state=tk.NORMAL if self.bboxes_on_local else tk.DISABLED)

    def label_slider_status_update(self):
        self.sliders.label_slider.configure(state=tk.NORMAL if self.labels_on_local else tk.DISABLED)

    def masks_slider_status_update(self):
        self.sliders.mask_slider.configure(state=tk.NORMAL if self.masks_on_local else tk.DISABLED)

    def bind_events(self):
        """Binds events.
        """
        # Navigation
        self.root.bind("<Left>", self.prev_img)
        self.root.bind("<k>", self.prev_img)
        self.root.bind("<Right>", self.next_img)
        self.root.bind("<j>", self.next_img)
        self.root.bind("<Control-q>", self.exit)
        self.root.bind("<Control-w>", self.exit)

        # Files
        self.root.bind("<Control-s>", self.save_image)

        # View Toggles
        self.root.bind("<b>", self.toggle_bboxes)
        self.root.bind("<Control-b>", self.toggle_bboxes)
        self.root.bind("<l>", self.toggle_labels)
        self.root.bind("<Control-l>", self.toggle_labels)
        self.root.bind("<m>", self.toggle_masks)
        self.root.bind("<Control-m>", self.toggle_masks)
        self.root.bind("<space>", self.toggle_all)

        # Objects Panel
        self.objects_panel.category_box.bind("<<ListboxSelect>>", self.select_category)
        self.objects_panel.object_box.bind("<<ListboxSelect>>", self.select_object)
        self.image_panel.bind("<Button-1>", lambda e: self.image_panel.focus_set())


def print_info(message: str):
    logging.info(message)


def main():
    print_info("Starting...")
    args = parser.parse_args()

    if not args.annotations:
        root = tk.Tk()
        root.title("COCO Viewer")
        root.geometry("300x150")  # app size when no data is provided
        messagebox.showwarning("Warning!", "Nothing to show.\nPlease specify a path to the COCO dataset!")
        print_info("Exiting...")
        root.destroy()
        return
        
    if not args.images:
        #auto search images
        args.images = input('Input for [Image Prefix Folder] >> ')
        if not os.path.isdir(args.images):
            _ = input("Error: show be a FOLDER. PRESS ANY TO EXIT!")
            return
        
    root = tk.Tk()
    root.title("COCO Viewer")
    data = Data(args.images, args.annotations)
    statusbar = StatusBar(root)
    sliders = SlidersBar(root)
    objects_panel = ObjectsPanel(root)
    menu = Menu(root)
    image_panel = ImagePanel(root)
    Controller(data, root, image_panel, statusbar, menu, objects_panel, sliders)
    root.mainloop()


if __name__ == "__main__":
    main()



#!/usr/bin/python
'''
Author: your name
Date: 2021-09-28 14:14:27
LastEditTime: 2021-10-13 19:04:19
LastEditors: Please set LastEditors
Description: In User Settings Edit
FilePath: \BCNete:\cxf\crop_video.py
'''
# !pyinstaller -F crop_image.py -i mypython.ico
# chenxinfeng
# ------使用方法------
# 直接拖动文件夹到EXE中
import sys
import os
from glob import glob 
from PIL import Image

'''
vim config_video.py
## CROP_VIDEO.EXE
# crop_xywh = [640*1,480*1, 640,480]
w,h = 800, 600
crop_xywh = [[w*0,h*0,w,h],
             [w*1,h*0,w,h],
             [w*2,h*0,w,h],
             [w*0,h*1,w,h],
             [w*1,h*1,w,h],
             [w*2,h*1,w,h]]

keepXeqY = False
## EXTRACT_FRAME.EXE
numframe_to_extract = 200
maxlength = 10000
'''
#xywh = [50,0,600,585]
#folder = r"E:\cxf\Videos\20210601 LS RAT"

def xywh2whxy(xywh, keepXeqY=True):
    if keepXeqY:
        maxXY = max(xywh[2:])
        xywh[2] = xywh[3] = maxXY
    whxy = (xywh[2], xywh[3], xywh[0], xywh[1])
    return whxy

def cropimage_to_view(filename, whxy, postfix=None):
    outfileformat = "{}_view_"+str(postfix)+".png" if postfix else "{}_view.png"
    outfilename = outfileformat.format(filename[:-4])
    im = Image.open(filename)
    left, top, right, bottom = whxy[2], whxy[3], whxy[0]+whxy[2], whxy[1]+whxy[3]
    im1 = im.crop((left, top, right, bottom))
    im1.save(outfilename)

if __name__ == '__main__':
    n = len(sys.argv)
    if n == 1:
        img = input("Choose a image: >> ")
        if img == None:
            exit()
        else:
            sys.argv.append(img)
            
    print(sys.argv[1:])
    
    # check input is file or folder
    file_or_folderpath = sys.argv[1]
    if os.path.isfile(file_or_folderpath):
        video_foldname, f =os.path.split(file_or_folderpath)
        os.chdir(video_foldname)
        filenamesList = [f]
    else:
        assert False, 'Input should be FILE or FOLDER'
        
    # read config_video.py
    import sys; sys.path.append('.')
    import config_video as config
    assert hasattr(config, 'crop_xywh'), 'Need [crop_xywh] in [config_video.py]'
    keepXeqY = getattr(config, 'keepXeqY', True)

    if type(config.crop_xywh[0]) == list:
        for i, crop_xywh in enumerate(config.crop_xywh):
            whxy = xywh2whxy(crop_xywh, keepXeqY)
            cropimage_to_view(filenamesList[0], whxy, i+1)
    else:
        whxy = xywh2whxy(config.crop_xywh, keepXeqY)
        cropimage_to_view(filenamesList[0], whxy)
        
    print("Succeed")


#!/usr/bin/python
'''
Author: your name
Date: 2021-09-28 14:14:27
LastEditTime: 2021-10-13 19:04:19
LastEditors: Please set LastEditors
Description: In User Settings Edit
FilePath: \BCNete:\cxf\crop_video.py
'''
# !pyinstaller -F crop_image.py -i mypython.ico
# chenxinfeng
# ------使用方法------
# 直接拖动文件夹到EXE中
import sys
import os
from glob import glob 
import cxfguilib as cg
from PIL import Image

'''
vim config_video.py
## CROP_VIDEO.EXE
# crop_xywh = [640*1,480*1, 640,480]
w,h = 800, 600
crop_xywh = [[w*0,h*0,w,h],
             [w*1,h*0,w,h],
             [w*2,h*0,w,h],
             [w*0,h*1,w,h],
             [w*1,h*1,w,h],
             [w*2,h*1,w,h]]

keepXeqY = False
## EXTRACT_FRAME.EXE
numframe_to_extract = 200
maxlength = 10000
'''
#xywh = [50,0,600,585]
#folder = r"E:\cxf\Videos\20210601 LS RAT"

def xywh2whxy(xywh, keepXeqY=True):
    if keepXeqY:
        maxXY = max(xywh[2:])
        xywh[2] = xywh[3] = maxXY
    whxy = (xywh[2], xywh[3], xywh[0], xywh[1])
    return whxy

def convert_folder_to_mp4(folder, whxy, postfix=None):
    os.chdir(folder)
    filenamesList = glob(r'*.jpg') +  glob(r'*.png')
    filenamesList = [f for f in filenamesList if 'output' not in f]
    
    # check filenameList is sorted. If not, rise an error!
    if len(filenamesList) == 0:
        print("Folder contain no JPG/JPEG/PNG images!")
        return

    # convert videos
    for filename in filenamesList:
        outfileformat = "{}_output_"+str(postfix)+".png" if postfix else "{}_output.png"
        outfilename = outfileformat.format(filename[:-4])
        im = Image.open(filename)
        left, top, right, bottom = whxy[2], whxy[3], whxy[0]+whxy[2], whxy[1]+whxy[3]
        im1 = im.crop((left, top, right, bottom))
        im1.save(outfilename)

if __name__ == '__main__':
    n = len(sys.argv)
    if n == 1:
        folder = cg.uigetfolder()
        if folder == None:
            exit()
        else:
            sys.argv.append(folder)
            
    print(sys.argv[1:])

    video_foldname = sys.argv[1]
    config = cg.getfoldconfigpy(video_foldname)
    assert hasattr(config, 'crop_xywh'), 'Need [crop_xywh] in [config_video.py]'
    keepXeqY = getattr(config, 'keepXeqY', True)

    if type(config.crop_xywh[0]) == list:
        for i, crop_xywh in enumerate(config.crop_xywh):
            whxy = xywh2whxy(crop_xywh, keepXeqY)
            convert_folder_to_mp4(video_foldname, whxy, i+1)
    else:
        whxy = xywh2whxy(config.crop_xywh, keepXeqY)
        convert_folder_to_mp4(video_foldname, whxy)
    print("Succeed")
 


#setup.py
from setuptools import setup, find_packages
setup(
    name='lilab', # 应用名
    version='0.1', # 版本号
    packages=find_packages(include=['LILAB*']), # 包括在安装包内的 Python 包
    author='chenxf',
    author_email='cxf529125853@163.com',
)



#!/usr/bin/python
'''
Author: your name
Date: 2021-10-06 13:15:24
LastEditTime: 2021-10-06 14:21:52
LastEditors: Please set LastEditors
Description: In User Settings Edit
FilePath: \BCNet\extract_JPG_from_dataset.py
'''
#!pyinstaller -F extract_JPG_from_dataset.py -i logo_coco.ico
# python extract_JPG_from_dataset.py E:\cxf\BCNet\data\coco_rat_val.json
import json
import sys
import shutil
import os

if __name__ == '__main__':
    n = len(sys.argv)
    assert n==2
          
    print(sys.argv[1:])
    cocofile = sys.argv[1]
    with open(cocofile, 'r') as f:
        data = json.load(f)
    imgs = data['images']
    files = [img['file_name'] for img in imgs]

    file0 = files[0]
    jsonfolder, jsonname = os.path.split(cocofile)
    if os.path.isfile(os.path.join(jsonfolder, file0)):
        img_prefix = jsonfolder
    else:
        currentlist = [os.path.join(jsonfolder, d) for d in os.listdir(jsonfolder)]
        currentfolder = [d for d in currentlist if os.path.isdir(d) and 'extract' not in d]
        for folder in currentfolder:
            if os.path.isfile(os.path.join(folder, file0)):
                img_prefix = folder
                break
        else:
            img_prefix = input("Please give [Image Prefix] >> ")

    assert os.path.isfile(os.path.join(img_prefix, file0))
    outfoldername = os.path.join(jsonfolder, 'extract_'+ os.path.splitext(jsonname)[0])
    os.makedirs(outfoldername, exist_ok=True)

    # copy file to output folder
    for file in files:
        shutil.copy(os.path.join(img_prefix, file), outfoldername)



'''
Author: your name
Date: 2021-10-03 13:47:29
LastEditTime: 2021-10-11 17:36:56
LastEditors: Please set LastEditors
Description: In User Settings Edit
FilePath: \cxf\merge_multipart_coco.py
'''
# !pyinstaller -F merge_multipart_coco.py -i merge_multipart_coco.ico
# chenxinfeng
# ------使用方法------
# 直接拖动json到EXE中
# merge parts of 'rat1', 'rat2' into 'rat's
#


# %% import
import json
import sys, os
import datetime
from pycocotools.coco import COCO

def convert(anno, anno_out):
    coco = COCO(anno)
    dataout = dict()
    dataout['categories'] = coco.dataset['categories']
    dataout['images'] = []
    dataout['annotations'] = []
    dataout['info'] = dict(
        description = 'labelme2posecoco', 
        version = '1.0', 
        year = datetime.datetime.today().year,
        date_created = datetime.datetime.today().strftime('%Y/%m/%d'))
    dataout['licences'] = ''
    dataout['images'] = coco.dataset['images']
    # %%
    annoid =0
    imgids = coco.getImgIds()
    catids = list(coco.cats.keys())
    for image_id in imgids:
        anns = coco.imgToAnns[image_id]
        allrat_seg = {catid:[] for catid in catids}

        for iann in anns:
            catid = iann['category_id']
            allrat_seg[catid] += iann['segmentation']

        for catid, segs in allrat_seg.items():
            if segs == []: continue
            segs_1d = [j for sub in segs for j in sub]
            x_min, y_min = min(segs_1d[::2]), min(segs_1d[1::2])
            x_max, y_max = max(segs_1d[::2]), max(segs_1d[1::2])
            bbox = [int(i) for i in [x_min, y_min, x_max-x_min, y_max-y_min]]
            annotations0 = dict(
                id = annoid,
                bbox = bbox,
                iscrowd = 0,
                area = bbox[2]*bbox[3],
                category_id = catid,
                image_id = image_id,
                segmentation = segs
            )
            dataout['annotations'].append(annotations0)
            annoid+=1

    with open(anno_out, 'w') as f:
        json.dump(dataout, f, indent=4)

if __name__ == "__main__":
    assert len(sys.argv)==2
    anno = sys.argv[1]
    assert os.path.isfile(anno)
    prefix =  os.path.splitext(anno)[0]
    anno_out = prefix + '_merge.json'
    print(anno)
    convert(anno, anno_out)